(function () {
    'use strict';

    var PluginContext = function () {
      function PluginContext() {
      }
      PluginContext.initialize = function (value) {
        return Object.assign(this.instance, value);
      };
      Object.defineProperty(PluginContext, 'instance', {
        get: function () {
          if (!this._instance) {
            this._instance = new PluginContext();
          }
          return this._instance;
        },
        enumerable: false,
        configurable: true
      });
      return PluginContext;
    }();

    var defaultPluginSetting = { codeMirrorBasePath: 'https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.59.4' };
    var pluginSetting = Object.assign({}, defaultPluginSetting);
    var initializePluginSetting = function (setting) {
      setting !== null && setting !== void 0 ? setting : setting = defaultPluginSetting;
      return Object.assign(pluginSetting, defaultPluginSetting, setting);
    };

    var pluginId = 'cnb-tiny-source-code';

    var CodeMirrorBuilder = function () {
      function CodeMirrorBuilder(editor, settings) {
        this.editor = editor;
        this.settings = settings;
      }
      Object.defineProperty(CodeMirrorBuilder.prototype, 'iframe', {
        get: function () {
          return document.querySelector('.tox-navobj iframe');
        },
        enumerable: false,
        configurable: true
      });
      Object.defineProperty(CodeMirrorBuilder.prototype, 'cmEditor', {
        get: function () {
          return this.iframe.contentWindow['cmEditor'];
        },
        enumerable: false,
        configurable: true
      });
      CodeMirrorBuilder.prototype.build = function () {
        return '\n    <!DOCTYPE html>\n    <html lang="zh">\n      <head>\n        '.concat(this.buildHead(), '\n      <title></title>\n      </head>\n      <body>\n      </body>\n    </html>\n    ');
      };
      CodeMirrorBuilder.prototype.buildHead = function () {
        return '\n    <base href="'.concat(this.editor.dom.encode(this.editor.documentBaseURI.getURI()), '">\n    <link type="text/css" rel="stylesheet" href="').concat(this.settings.codeMirrorBasePath, '/codemirror.min.css">\n    <link rel="stylesheet" href="').concat(this.settings.codeMirrorBasePath, '/addon/fold/foldgutter.css">\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/codemirror.min.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/addon/fold/foldcode.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/addon/fold/xml-fold.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/addon/fold/foldgutter.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/addon/edit/closetag.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/addon/edit/matchtags.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/addon/edit/matchbrackets.js"></script>\n\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/mode/vbscript/vbscript.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/mode/javascript/javascript.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/mode/css/css.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/mode/xml/xml.js"></script>\n    <script type="text/javascript" src="').concat(this.settings.codeMirrorBasePath, '/mode/htmlmixed/htmlmixed.js"></script>\n\n    ').concat(CodeMirrorBuilder.buildCodeMirrorInitializeScript(), '\n    ').concat(CodeMirrorBuilder.buildStyle());
      };
      CodeMirrorBuilder.buildCodeMirrorInitializeScript = function () {
        return '\n    <script type="text/javascript">\n      window.onload = function() {\n        window.cmEditor = CodeMirror(document.body, {\n          value: "",\n          lineNumbers: true,\n          lineWrapping: true,\n          indentUnit: 4,\n          indentWithTabs: false,\n          mode: \'text/html\',\n          gutters: ["CodeMirror-linenumbers", "CodeMirror-foldgutter"],\n          foldGutter: true,\n          foldOptions: {\n            widget: \'...\'\n          },\n          matchBrackets: true,\n          autoCloseTags: true,\n          matchTags: { bothTags: true },\n          extraKeys: {\n            Tab: function (cm) {\n              if (cm.somethingSelected()) {\n                cm.indentSelection("add");\n              } else {\n                cm.replaceSelection(\n                  cm.getOption("indentWithTabs") ?\n                    "\\t":\n                    Array(cm.getOption("indentUnit") + 1).join(" "), "end", "+input");\n              }\n            }\n          }\n        });\n      }\n    </script>\n    ';
      };
      CodeMirrorBuilder.buildStyle = function () {
        return '\n    <style>\n      * {\n        box-sizing: border-box;\n      }\n      html,\n      body,\n      .CodeMirror {\n        height: 100%;\n        font-size: 16px;\n      }\n      body {\n        margin: 0;\n      }\n      .CodeMirror {\n        border: 1px solid #ddd;\n        border-radius: 3px;\n      }\n    </style>\n    ';
      };
      return CodeMirrorBuilder;
    }();

    var open = function () {
      var _a = PluginContext.instance, editor = _a.editor, editorBuilder = _a.codeMirrorBuilder;
      initializePluginSetting(editor.settings[pluginId]);
      var sourceHtml = editor.getContent({ source_view: true });
      editor.windowManager.open({
        title: '\u6e90\u4ee3\u7801',
        size: 'large',
        body: {
          type: 'panel',
          items: [{
              type: 'iframe',
              name: 'cmHtmlEditor',
              sandboxed: true
            }]
        },
        buttons: [
          {
            type: 'cancel',
            name: 'cancel',
            text: '\u53d6\u6d88'
          },
          {
            type: 'submit',
            name: 'save',
            text: '\u4fdd\u5b58',
            primary: true
          }
        ],
        initialData: { cmHtmlEditor: editorBuilder.build() },
        onSubmit: function (api) {
          var html = editorBuilder.cmEditor.getValue();
          editor.focus();
          editor.undoManager.transact(function () {
            editor.setContent(html);
          });
          editor.selection.setCursorLocation();
          editor.nodeChanged();
          api.close();
        }
      });
      editorBuilder.iframe.onload = function () {
        editorBuilder.cmEditor.setValue(sourceHtml);
      };
    };

    var iconId = 'icon-'.concat(pluginId);
    var registerIcon = function () {
      var editor = PluginContext.instance.editor;
      editor.ui.registry.addIcon(iconId, '<svg width="20" height="20" viewBox="0 0 1024 1024" xmlns="http://www.w3.org/2000/svg"><path d="M194.33 479.765h-101.295v-101.28h-46.035v267.030h46.035v-119.7h101.28v119.7h46.035v-267.030h-46.035v101.28zM277.205 415.31h73.665v230.205h46.035v-230.205h73.665v-36.825h-193.365v36.825zM627.815 560.42l-46.755-181.935h-73.665v267.030h46.035v-211.785l46.035 211.785h55.245l46.83-213.93-0.795 213.93h46.035v-267.030h-73.665l-45.3 181.935zM848.090 608.69v-230.205h-46.035v267.030h174.945v-36.825h-128.91z"></path></svg>');
    };
    var setup$1 = function () {
      var editor = PluginContext.instance.editor;
      registerIcon();
      editor.ui.registry.addToggleButton(pluginId, {
        icon: iconId,
        tooltip: '\u6e90\u4ee3\u7801',
        onAction: function () {
          open();
        }
      });
    };
    var Button = function () {
      function Button() {
      }
      Button.setup = setup$1;
      return Button;
    }();

    var setup = function (editor) {
      var setting = initializePluginSetting(editor.settings[pluginId]);
      PluginContext.initialize({
        setting: setting,
        codeMirrorBuilder: new CodeMirrorBuilder(editor, setting),
        editor: editor
      });
      Button.setup();
    };
    function Plugin () {
      tinymce.PluginManager.add('cnb-tiny-source-code', setup);
    }

    Plugin();

})();
